#   -*- coding: utf-8 -*-
#
#   This file is part of SKALE.py
#
#   Copyright (C) 2019-Present SKALE Labs
#
#   SKALE.py is free software: you can redistribute it and/or modify
#   it under the terms of the GNU Affero General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   SKALE.py is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU Affero General Public License for more details.
#
#   You should have received a copy of the GNU Affero General Public License
#   along with SKALE.py.  If not, see <https://www.gnu.org/licenses/>.

# flake8: noqa
CONSTELLATIONS = [
    'Andromeda',
    'Antlia',
    'Apus',
    'Aquarius',
    'Aquila',
    'Ara',
    'Aries',
    'Auriga',
    'Caelum',
    'Camelopardalis',
    'Cancer',
    'Canes Venatici',
    'Canis Major',
    'Canis Minor',
    'Capricornus',
    'Carina',
    'Cassiopeia',
    'Centaurus',
    'Cepheus',
    'Cetus',
    'Chamaeleon',
    'Circinus',
    'Columba',
    'Coma Berenices',
    'Corona Austrina',
    'Corona Borealis',
    'Corvus',
    'Crater',
    'Crux',
    'Cygnus',
    'Delphinus',
    'Dorado',
    'Draco',
    'Equuleus',
    'Eridanus',
    'Fornax',
    'Gemini',
    'Grus',
    'Hercules',
    'Horologium',
    'Hydra',
    'Hydrus',
    'Indus',
    'Lacerta',
    'Leo',
    'Leo Minor',
    'Lepus',
    'Libra',
    'Lupus',
    'Lynx',
    'Lyra',
    'Mensa',
    'Microscopium',
    'Monoceros',
    'Musca',
    'Norma',
    'Octans',
    'Ophiuchus',
    'Orion',
    'Pavo',
    'Pegasus',
    'Perseus',
    'Phoenix',
    'Pictor',
    'Pisces',
    'Piscis Austrinus',
    'Puppis',
    'Pyxis',
    'Reticulum',
    'Sagitta',
    'Sagittarius',
    'Scorpius',
    'Sculptor',
    'Scutum',
    'Serpens',
    'Sextans',
    'Taurus',
    'Telescopium',
    'Triangulum',
    'Triangulum Australe',
    'Tucana',
    'Ursa Major',
    'Ursa Minor',
    'Vela',
    'Virgo',
    'Volans',
    'Vulpecula',
]
STARS = [
    'ACAMAR',
    'ACHERNAR',
    'Achird',
    'ACRUX',
    'Acubens',
    'ADARA',
    'Adhafera',
    'Adhil',
    'AGENA',
    'Ain al Rami',
    'Ain',
    'Al Anz',
    'Aladfar',
    'Alathfar',
    'Albaldah',
    'Albali',
    'ALBIREO',
    'Alchiba',
    'ALCOR',
    'ALCYONE',
    'ALDEBARAN',
    'ALDERAMIN',
    'Aldhibah',
    'Alfecca Meridiana',
    'Alfirk',
    'ALGENIB',
    'ALGIEBA',
    'ALGOL',
    'Algorab',
    'ALHENA',
    'ALIOTH',
    'ALKAID',
    'Alkalurops',
    'Alkes',
    'Alkurhah',
    'ALMAAK',
    'ALNAIR',
    'ALNATH',
    'ALNILAM',
    'ALNITAK',
    'Alniyat',
    'Alniyat',
    'ALPHARD',
    'ALPHEKKA',
    'ALPHERATZ',
    'Alrai',
    'Alrisha',
    'Alsafi',
    'Alsciaukat',
    'ALSHAIN',
    'Alshat',
    'Alsuhail',
    'ALTAIR',
    'Altarf',
    'Alterf',
    'Aludra',
    'Alula Australis',
    'Alula Borealis',
    'Alya',
    'Alzirr',
    'Ancha',
    'Angetenar',
    'ANKAA',
    'Anser',
    'ANTARES',
    'ARCTURUS',
    'Arkab Posterior',
    'Arkab Prior',
    'ARNEB',
    'Arrakis',
    'Ascella',
    'Asellus Australis',
    'Asellus Borealis',
    'Asellus Primus',
    'Asellus Secondus',
    'Asellus Tertius',
    'Asterope',
    'Atik',
    'Atlas',
    'Auva',
    'Avior',
    'Azelfafage',
    'Azha',
    'Azmidiske',
    'Baham',
    'Baten Kaitos',
    'Becrux',
    'Beid',
    'BELLATRIX',
    'BETELGEUSE',
    'Botein',
    'Brachium',
    'CANOPUS',
    'CAPELLA',
    'Caph',
    'CASTOR',
    'Cebalrai',
    'Celaeno',
    'Chara',
    'Chort',
    'COR CAROLI',
    'Cursa',
    'Dabih',
    'Deneb Algedi',
    'Deneb Dulfim',
    'Deneb el Okab',
    'DENEB',
    'DENEBOLA',
    'Dheneb',
    'Diadem',
    'DIPHDA',
    'Double Double',
    'Dschubba',
    'Dsiban',
    'DUBHE',
    'Ed Asich',
    'Electra',
    'ELNATH',
    'ENIF',
    'ETAMIN',
    'FOMALHAUT',
    'Fornacis',
    'Fum al Samakah',
    'Furud',
    'Gacrux',
    'Gianfar',
    'Gienah Cygni',
    'Gienah Ghurab',
    'Gomeisa',
    'Gorgonea Quarta',
    'Gorgonea Secunda',
    'Gorgonea Tertia',
    'Graffias',
    'Grafias',
    'Grumium',
    'HADAR',
    'Haedi',
    'HAMAL',
    'Hassaleh',
    'Head of Hydrus',
    'Heze',
    'Hoedus',
    'Homam',
    'Hyadum',
    'IZAR',
    'Jabbah',
    'Kaffaljidhma',
    'Kajam',
    'KAUS AUSTRALIS',
    'Kaus Borealis',
    'Kaus Meridionalis',
    'Keid',
    'Kitalpha',
    'KOCAB',
    'Kornephoros',
    'Kraz',
    'Kuma',
    'Lesath',
    'Maasym',
    'Maia',
    'Marfak',
    'Marfak',
    'Marfic',
    'Marfik',
    'MARKAB',
    'Matar',
    'Mebsuta',
    'MEGREZ',
    'Meissa',
    'Mekbuda',
    'Menkalinan',
    'MENKAR',
    'Menkar',
    'Menkent',
    'Menkib',
    'MERAK',
    'Merga',
    'Merope',
    'Mesarthim',
    'Metallah',
    'Miaplacidus',
    'Minkar',
    'MINTAKA',
    'MIRA',
    'MIRACH',
    'Miram',
    'MIRPHAK',
    'MIZAR',
    'Mufrid',
    'Muliphen',
    'Murzim',
    'Muscida',
    'Muscida',
    'Muscida',
    'Naos',
    'Nash',
    'Nashira',
    'Nekkar',
    'NIHAL',
    'Nodus Secundus',
    'NUNKI',
    'Nusakan',
    'Peacock',
    'PHAD',
    'Phaet',
    'Pherkad Minor',
    'Pherkad',
    'Pleione',
    'Polaris Australis',
    'POLARIS',
    'POLLUX',
    'Porrima',
    'Praecipua',
    'Prima Giedi',
    'PROCYON',
    'Propus',
    'Propus',
    'Propus',
    'Rana',
    'RASALGETHI',
    'RASALHAGUE',
    'Rastaban',
    'REGULUS',
    'Rigel Kentaurus',
    'RIGEL',
    'Rijl al Awwa',
    'Rotanev',
    'Ruchba',
    'Ruchbah',
    'Rukbat',
    'Sabik',
    'Sadalachbia',
    'SADALMELIK',
    'Sadalsuud',
    'Sadr',
    'SAIPH',
    'Salm',
    'Sargas',
    'Sarin',
    'Sceptrum',
    'SCHEAT',
    'Secunda Giedi',
    'Segin',
    'Seginus',
    'Sham',
    'Sharatan',
    'SHAULA',
    'SHEDIR',
    'Sheliak',
    'SIRIUS',
    'Situla',
    'Skat',
    'SPICA',
    'Sterope',
    'Sualocin',
    'Subra',
    'Sulafat',
    'Syrma',
    'Tabit',
    'Talitha',
    'Tania Australis',
    'Tania Borealis',
    'TARAZED',
    'Taygeta',
    'Tegmen',
    'Tejat Posterior',
    'Terebellum',
    'Thabit',
    'Theemim',
    'THUBAN',
    'Turais',
    'Tyl',
    'UNUKALHAI',
    'VEGA',
    'VINDEMIATRIX',
    'Wasat',
    'Wezen',
    'Wezn',
    'Yed Posterior',
    'Yed Prior',
    'Yildun',
    'Zaniah',
    'Zaurak',
    'Zavijah',
    'Zibal',
    'Zosma',
    'Zuben Elakrab',
]

POSITIVE_AND_TIME_ADJECTIVES = [
    'aggressive',
    'agreeable',
    'ambitious',
    'brave',
    'calm',
    'delightful',
    'eager',
    'faithful',
    'gentle',
    'happy',
    'jolly',
    'kind',
    'lively',
    'nice',
    'obedient',
    'polite',
    'proud',
    'silly',
    'thankful',
    'victorious',
    'witty',
    'wonderful',
    'zealous',
    'ancient',
    'brief',
    'early',
    'fast',
    'future',
    'late',
    'long',
    'modern',
    'old',
    'old-fashioned',
    'prehistoric',
    'quick',
    'rapid',
    'short',
    'slow',
    'swift',
    'young',
]
SOUND_AND_APPEARANCE_ADJECTIVES = [
    'deafening',
    'echoing',
    'faint',
    'harsh',
    'hissing',
    'howling',
    'loud',
    'melodic',
    'noisy',
    'purring',
    'quiet',
    'rapping',
    'raspy',
    'rhythmic',
    'screeching',
    'shrilling',
    'squeaking',
    'thundering',
    'tinkling',
    'wailing',
    'whining',
    'whispering',
    'attractive',
    'beautiful',
    'chubby',
    'clean',
    'dazzling',
    'drab',
    'elegant',
    'fancy',
    'fit',
    'glamorous',
    'gorgeous',
    'handsome',
    'long',
    'magnificent',
    'plain',
    'plump',
    'quaint',
    'scruffy',
    'shapely',
    'stocky',
]
